<?php

declare(strict_types=1);

namespace Drupal\Tests\crowd\Functional;

use Drupal\Core\Test\AssertMailTrait;
use Drupal\crowd\Crowd\CrowdConnectorInterface;
use Drupal\crowd\Form\SettingsForm;
use Drupal\Tests\BrowserTestBase;
use Drupal\user\Entity\Role;
use Drupal\user\RoleInterface;
use Drupal\user\UserInterface;

/**
 * Defines a base test class for crowd tests.
 */
abstract class CrowdTestBase extends BrowserTestBase {

  use AssertMailTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'user',
    'crowd',
    'crowd_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Verified role.
   *
   * @var \Drupal\user\RoleInterface
   */
  protected RoleInterface $verifiedRole;

  /**
   * Restricted domain.
   *
   * @var string
   */
  protected string $restrictedDomain;

  /**
   * Connector.
   *
   * @var \Drupal\crowd\Crowd\CrowdConnectorInterface
   */
  protected CrowdConnectorInterface $crowdConnector;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->verifiedRole = Role::create([
      'id' => $this->randomMachineName(),
      'label' => $this->randomMachineName(),
    ]);
    $this->verifiedRole->save();
    $this->restrictedDomain = sprintf('%s.dev.null', $this->randomMachineName());
    $this->config(SettingsForm::CROWD_SETTINGS)
      ->set('username', $this->randomMachineName())
      ->set('server_uri', sprintf('https://%s.dev.null/crowd', mb_strtolower($this->randomMachineName())))
      ->set('email_as_username', TRUE)
      ->set('verified_role', $this->verifiedRole->id())
      ->set('restricted_domains', [$this->restrictedDomain])
      ->save();
    $this->config('user.settings')
      ->set('verify_mail', TRUE)
      ->set('register', UserInterface::REGISTER_VISITORS)
      ->save();
    $this->crowdConnector = \Drupal::service('crowd.connector');
  }

  /**
   * Generates a random email address.
   *
   * @return string
   *   Random email.
   */
  protected function randomEmail(): string {
    return sprintf('%s@%s.dev.null', mb_strtolower($this->randomMachineName()), mb_strtolower($this->randomMachineName()));
  }

  /**
   * {@inheritdoc}
   */
  protected function getResetUrl(array $mail): ?string {
    $urls = [];
    preg_match('#.+user/reset/.+#', $mail['body'], $urls);
    return $urls[0] ?? NULL;
  }

}
